@@ -79,8 +79,6 @@ class BinaryClassificationDataset(Sequence):
         self.batch_size = batch_size
         self.augmentations = augmentations
 
-        self.counter = 0
-
 
     def __len__(self):
         return len(self.data) // self.batch_size
@@ -89,11 +87,6 @@ class BinaryClassificationDataset(Sequence):
         random.shuffle(self.data)        
 
     def __getitem__(self, idx):
-        self.counter += 1
-        if self.counter == self.__len__() + 1:
-            self.counter = 0
-            self.shuffle()
-
         data_tuples = self.data[idx * self.batch_size:(idx + 1) * self.batch_size]
 
         batch_x = [i[0] for i in data_tuples]
@@ -126,21 +119,105 @@ class BinaryClassificationDataset(Sequence):
             for path in self.fail_filepaths:
                 imgs = cv2.imreadmulti(path)[1]
                 for i in self.tiff_layer:
-                    if imgs[0].ndim == 2:
-                        self.fail_images.append(imgs[i])
-                    else:
-                        self.fail_images.append(imgs[i][:,:,0])    
-
+                    self.fail_images.append(imgs[i][:,:,0])           
             for path in self.pass_filepaths:
                 imgs = cv2.imreadmulti(path)[1]
                 for i in self.tiff_layer:
-                    if imgs[0].ndim == 2:
-                        self.pass_images.append(imgs[i])
-                    else:
-                        self.pass_images.append(imgs[i][:,:,0])  
+                    self.pass_images.append(imgs[i][:,:,0])
 
 
 
         else:
             self.fail_images = [cv2.imread(path, 0) for path in self.fail_filepaths]
-            self.pass_images = [cv2.imread(path, 0) for path in self.pass_filepaths]
\ No newline at end of file
+            self.pass_images = [cv2.imread(path, 0) for path in self.pass_filepaths]
+
+
+class BalancedBinaryClassificationDataset(Sequence):
+    def __init__(self, base_dir = './data', fail_folder = 'Fail', pass_folder = 'Pass', 
+                 targets = [1,0], batch_size = 32, augmentations = None, max_files = 100,
+                 file_type = 'tiff', tiff_layers = None, json_shape = (1,17,363), dtype = np.float32):
+        
+        self.max_files = max_files
+        self.batch_size = batch_size
+        self.json_shape = json_shape
+        self.targets = targets
+        self.augmentations = augmentations
+        self.dtype = dtype
+
+        
+        self.fail_dir = os.path.join(os.path.abspath(base_dir), fail_folder)
+        self.pass_dir = os.path.join(os.path.abspath(base_dir), pass_folder)
+
+        self.fail_paths = [os.path.join(self.fail_dir, filename) for filename in os.listdir(self.fail_dir) if filename.endswith(file_type)]
+        self.pass_paths = [os.path.join(self.pass_dir, filename) for filename in os.listdir(self.pass_dir) if filename.endswith(file_type)]
+
+        if file_type in ['.tiff', '.tif', 'tiff', 'tif']:
+            self.fail_images, self.pass_images = self.read_images_tiff()
+        elif file_type in ['.json', 'json']:
+            self.fail_images, self.pass_images = self.read_images_json()
+        else: raise Exception('Unknown file_type')
+
+        self.batches = self.create_batches()
+        
+    def __len__(self):
+        # return the number of batches
+        return (self.max_files * 2) // self.batch_size
+
+    def __getitem__(self, idx):
+        batch = self.batches[idx]
+
+        images, labels = [], []
+        for image, label in batch:
+            if self.augmentations:
+                images.append(self.augmentations(image=image[0].astype(self.dtype))["image"])
+            else: 
+                images.append(image)
+
+            labels.append(label)
+
+        return np.array(images).astype(self.dtype), np.array(labels).astype(self.dtype)
+
+    def on_epoch_end(self):
+        self.batches = self.create_batches()
+
+    def read_images_tiff(self):
+        raise Exception('Not implemented yet')
+
+    def read_images_json(self):
+        channels, height, width = self.json_shape
+
+        fail_images = []
+        for path in self.fail_paths:
+            with open(path, 'r') as f:
+                data = np.array(json.load(f))
+            fail_images.append(data.reshape((channels, height, width)))
+
+        pass_images = []
+        for path in self.pass_paths:
+            with open(path, 'r') as f:
+                data = np.array(json.load(f))
+            pass_images.append(data.reshape((channels, height, width)))
+
+        return fail_images, pass_images
+    
+    def create_batches(self):
+        batches = []
+
+        fail_idxs = np.random.choice(range(len(self.fail_paths)), self.max_files, replace=False)
+        pass_idxs = np.random.choice(range(len(self.pass_paths)), self.max_files, replace=False)
+
+        epoch_fail_images = [(self.fail_images[i], self.targets[0]) for i in fail_idxs]
+        epoch_pass_images = [(self.pass_images[i], self.targets[1]) for i in pass_idxs]
+
+        epoch_data = epoch_fail_images + epoch_pass_images
+        random.shuffle(epoch_data)
+
+        start, end = 0, self.batch_size
+        for batch_idx in range(self.__len__()):
+            batch_data = epoch_data[start : end]
+            start += self.batch_size
+            end += self.batch_size
+            batches.append(batch_data)
+
+        return batches
+
